var RnaNormalize = function()
{
	var normalize = {};
	
	var selectedRna;
	var subtypeRna = null;
	
	var autoGoNext = 0;
	var newRna = [];
	
	// Page initialization
	normalize.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', normalize.validateStep1);
		Buttons.addClickHandler('btnSelectRna', normalize.selectRna);
		Events.addEventHandler('rna', 'base-selected', normalize.onRnaSelected);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', normalize.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', normalize.validateStep2);

		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', normalize.initializeStep3);
		Events.addEventHandler('normalizationDate', 'blur', Wizard.validateDate);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goprint', normalize.printVersion);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', normalize.submit);

		var rna = Data.json('page-data', 'rna-id');
		if (rna.length > 0)
		{
			Doc.show('step-1');
			autoGoNext = rna.length;
			// Get more information about the selected rna
			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRNAInfoForNormalize&rnaIds=' + rna.join(',');
			Wizard.asyncJsonRequest(url, normalize.onRnaInfoLoaded);
		}
		else
		{
			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRNAToNormalize';
			Wizard.showLoadingAnimation('Loading RNA extracts...');
			Wizard.asyncJsonRequest(url, normalize.onRnaLoaded);
		}
	}
	
	normalize.onRnaLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var rnaExtracts = response.rnaExtracts;
		
		rnaExtracts.sort(normalize.sortByQiacubePosition);
		
		if (rnaExtracts.length > 0)
		{
			var qiacubeRun = rnaExtracts[0].QiacubeDate + '#' + rnaExtracts[0].QiacubeRunNo;
			for (var i=0; i < rnaExtracts.length; i++)
			{
				var rna = rnaExtracts[i];
				var selected = (rna.QiacubeDate + '#' + rna.QiacubeRunNo) == qiacubeRun;
				normalize.addRnaOption(frm.rna, rna, selected);
			}
		}
		else
		{	
			Wizard.setInputStatus('rna', 'invalid', 'No RNA available for normalization.');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	/**
		Sort by qiacube date, run number and position
	*/
	normalize.sortByQiacubePosition = function(a, b)
	{
		if (a.QiacubeDate == b.QiacubeDate)
		{
			if (a.QiacubeRunNo == b.QiacubeRunNo)
			{
				return a.QiacubePosition-b.QiacubePosition;
			}
			else
			{
				return a.QiacubeRunNo - b.QiacubeRunNo;
			}
		}
		else
		{
			return a.QiacubeDate < b.QiacubeDate ? -1 : 1;
		}

		return a.id - b.id;
	}


	/**
		Sort by qiacube date, run number and position
	*/
	normalize.sortByStorageLocation = function(a, b)
	{
		if (a.bioWell.bioPlate.name == b.bioWell.bioPlate.name)
		{
			return a.bioWell.location < b.bioWell.location ? -1 : 1;
		}
		else
		{
			return a.bioWell.bioPlate.name < b.bioWell.bioPlate.name ? -1 : 1;
		}
	
		return a.id - b.id;
	}

	
	normalize.selectRna = function()
	{
		var frm = document.forms['reggie'];
		if (frm.rna.disabled) return;
		
		if (subtypeRna == null)
		{
			subtypeRna = Reggie.getSubtypeInfo('RNA');
		}
		
		newRna = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeRna.id;
		Dialogs.selectItem('EXTRACT', 'rna', 1, url);
	}

	normalize.onRnaSelected = function(event)
	{
		var rna = event.detail;
		
		var frm = document.forms['reggie'];
		var rnaList = frm.rna;
		var isNew = true;
		for (var i = 0; i < rnaList.length; i++)
		{
			if (rnaList[i].value == rna.id)
			{
				rnaList[i].selected = true;
				isNew = false;
			}
		}
		
		if (isNew) newRna[newRna.length] = rna.id;
		
		if (event.detail.remaining == 0 && newRna.length > 0)
		{
			// Get more information about the selected rna
			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRNAInfoForNormalize&rnaIds=' + newRna.join(',');
			Wizard.asyncJsonRequest(url, normalize.onRnaInfoLoaded);
		}
	}

	normalize.onRnaInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var rnaExtracts = response.rnaExtracts;
		rnaExtracts.sort(normalize.sortByQiacubePosition);
		
		for (var rnaNo = 0; rnaNo < rnaExtracts.length; rnaNo++)
		{
			var rna = rnaExtracts[rnaNo];
			normalize.addRnaOption(frm.rna, rna, true);
		}
		
		if (frm.rna.length == autoGoNext)
		{
			Wizard.goNext(true);
		}
	}
	
	normalize.addRnaOption = function(list, rna, selected)
	{
		var name = rna.name;
		if (rna.bioWell)
		{
			name += ' -- ' + Reggie.reformatDate(rna.QiacubeDate) + '; Run ' + rna.QiacubeRunNo + '; Pos ' + rna.QiacubePosition;
		}

		var option = new Option(name, rna.id, selected, selected);
		option.rna = rna;
		list[list.length] = option;
	}
	
	normalize.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		var rnaList = frm.rna;
		
		selectedRna = [];
		for (var i=0; i < rnaList.length; i++)
		{
			if (rnaList[i].selected)
			{
				selectedRna[selectedRna.length] = rnaList[i].rna;
			}
		}
			
		if (selectedRna.length == 0)
		{
			Wizard.setInputStatus('rna', 'invalid', 'Not selected', 'invalid');
			event.preventDefault();
		}
		else
		{
			Wizard.setInputStatus('rna', 'valid');
		}
	}


	normalize.initializeStep2 = function()
	{
		// Check if all RNA from same Qiacube run
		var qiacubeRun = selectedRna[0].QiacubeDate + '#' + selectedRna[0].QiacubeRunNo;
		var sameQiacubeRun = true;
		for (var i=1; i < selectedRna.length; i++)
		{
			var rna = selectedRna[i];
			if (qiacubeRun != rna.QiacubeDate+'#'+rna.QiacubeRunNo)
			{
				sameQiacubeRun = false;
				break;
			}
		}
		
		if (sameQiacubeRun)
		{
			selectedRna.sort(normalize.sortByQiacubePosition);
			Doc.show('qiacube-info');
			Doc.element('qiacube-date').innerHTML = Reggie.reformatDate(selectedRna[0].QiacubeDate);
			Doc.element('qiacube-run').innerHTML = selectedRna[0].QiacubeRunNo;
		}
		else
		{
			selectedRna.sort(normalize.sortByStorageLocation);
		}
		
		var html = '<table id="rnaTable">';
		html += '<thead>';
		// First header row
		html += '<tr>';
		html += '<th>RNA</th>';
		if (sameQiacubeRun)
		{
			html += '<th>Qiacube</th>';
		}
		else
		{
			html += '<th>Storage</th>';
		}
		html += '<th>NDConc</th>';
		html += '<th>Remain</th>';	
		html += '<th colspan="2">Volumes</th>';
		html += '<th></th>';
		html += '</tr>';
		// Second header row
		html += '<tr>';
		html += '<th></th>';
		if (sameQiacubeRun)
		{
			html += '<th>position</th>';
		}
		else
		{
			html += '<th>box</th>';
		}
		html += '<th>(ng/µl)</th>';
		html += '<th>(µg)</th>';
		html += '<th>(µl)</th>';
		html += '<th>H<sub>2</sub>O</th>';
		html += '<th>Remarks</th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var expectedQiacubePosition = 0;
		for (var i=0; i < selectedRna.length; i++)
		{
			var rna = selectedRna[i];
			if (sameQiacubeRun)
			{
				expectedQiacubePosition++;
				while (expectedQiacubePosition != rna.QiacubePosition && expectedQiacubePosition <= 12)
				{
					html += normalize.insertEmpty(expectedQiacubePosition);
					expectedQiacubePosition++;
				}
			}
			
			rna.usedQuantity = 1.1; // µg
			rna.volume = Math.ceil(10000*rna.usedQuantity/rna.NDConc) / 10;
			rna.water = 50-rna.volume;
			
			var notEnoughQuantity = rna.remainingQuantity < rna.usedQuantity;
			
			var hiSenseChecked = '';
			var ndConc = null;
			html += '<tr class="highlight">';
			html += '<td class="rna">'+Strings.encodeTags(rna.nextAliquotName)+'</td>';
			if (sameQiacubeRun)
			{
				html += '<td class="qiacubepos">'+rna.QiacubePosition+'</td>';
			}
			else
			{
				html += '<td class="box">'+Strings.encodeTags(rna.bioWell.bioPlate.name + ' ' + rna.bioWell.location)+'</td>';
			}
			html += '<td class="ndconc">'+ Numbers.formatNumber(rna.NDConc, 2) + '</td>';
			html += '<td class="remain">'+ Numbers.formatNumber(rna.remainingQuantity, 2) + '</td>';
			html += '<td class="volume">'+(notEnoughQuantity ? '' : Numbers.formatNumber(rna.volume, 1))+'</td>';
			html += '<td class="water">'+(notEnoughQuantity ? '' : Numbers.formatNumber(rna.water, 1))+'</td>';
			html += '<td class="remarks">'+(notEnoughQuantity ? 'Not enough quantity' : '')+'</td>';
			html += '</tr>'
		}
		if (sameQiacubeRun)
		{
			expectedQiacubePosition++;
			while (expectedQiacubePosition <= 12)
			{
				html += normalize.insertEmpty(expectedQiacubePosition);
				expectedQiacubePosition++;
			}
		}
		html += '</tbody>';
		html += '</table>';
		
		Doc.element('rnaNormalize').innerHTML = html;
		Wizard.setCurrentStep(2);
		Doc.show('gonext');
		Doc.show('goprint');
		Doc.show('gocancel');
		Wizard.showGoNextConfirmation(true, 'Print the lab protocol before continuing with the registration!');
		Wizard.keepSessionAlive();
	}
	
	normalize.insertEmpty = function(qiacubePosition)
	{
		var html = '<tr class="highlight empty">';
		html += '<td class="rna"></td>';
		html += '<td class="qiacubepos">'+qiacubePosition+'</td>';
		html += '<td class="ndconc"></td>';
		html += '<td class="remain"></td>';
		html += '<td class="volume"></td>';
		html += '<td class="water"></td>';
		html += '<td class="remarks">Not enough quantity</td>';
		html += '</tr>'
		return html;
	}
	
	normalize.printVersion = function()
	{
		Wizard.hideGoNextConfirmation();
		Doc.removeClass('gonext', 'disabled');
		var frm = document.forms['reggie'];
		var reportName = 'RNA Normalization Protocol';
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.';
		Reggie.openPrintWindow('full-protocol', reportName, 'portrait', printNote, '../', 'normalize_rna.css');
	}

	normalize.validateStep2 = function(event)
	{}

	normalize.initializeStep3 = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setCurrentStep(3);
		Doc.show('goregister');
		Doc.show('gocancel');
		Events.sendChangeEvent('normalizationDate');
		frm.normalizationDate.focus();
	}
	
	normalize.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		submitInfo.normalizationDate = frm.normalizationDate.value;
		submitInfo.normalizationOperator = frm.normalizationOperator.value;
		
		var rnaAliquots = [];
		for (var i=0; i < selectedRna.length; i++)
		{
			var rna = selectedRna[i];
			if (rna.usedQuantity < rna.remainingQuantity)
			{
				var normalizedRna = {};
				normalizedRna.parentId = rna.id;
				normalizedRna.name = rna.nextAliquotName;

				normalizedRna.usedQuantity = rna.usedQuantity; // µg
				normalizedRna.NDConc = 1000 * rna.usedQuantity / (rna.volume+rna.water); // ng/µl
				
				rnaAliquots[rnaAliquots.length] = normalizedRna;
			}
		}
		submitInfo.rnaAliquots = rnaAliquots;

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterNormalizedRNA';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, normalize.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	normalize.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return normalize;
}();

Doc.onLoad(RnaNormalize.initPage);

